@@ -118,13 +118,13 @@ When live editing the filter, it is bound to :live.")
 (defun elfeed-search-last-entry ()
   "Place point on last entry."
   (interactive)
-  (setf (point) (point-max))
+  (goto-char (point-max))
   (forward-line -1))
 
 (defun elfeed-search-first-entry ()
   "Place point on first entry."
   (interactive)
-  (setf (point) (point-min)))
+  (goto-char (point-min)))
 
 (defvar elfeed-search-mode-map
   (let ((map (make-sparse-keymap)))
@@ -217,7 +217,15 @@ When live editing the filter, it is bound to :live.")
         truncate-lines t
         buffer-read-only t
         desktop-save-buffer #'elfeed-search-desktop-save
-        header-line-format '(:eval (funcall elfeed-search-header-function)))
+        ;; Provide format string via symbol value slot so that it will
+        ;; not be %-construct interpolated. The symbol is uninterned
+        ;; so that it's not *really* a global variable.
+        header-line-format
+        (let ((symbol (make-symbol "dummy")))
+          (put symbol 'risky-local-variable t)
+          `(:eval
+            (prog1 ',symbol
+              (set ',symbol (funcall elfeed-search-header-function))))))
   (set (make-local-variable 'bookmark-make-record-function)
        #'elfeed-search-bookmark-make-record)
   (buffer-disable-undo)
@@ -766,16 +774,21 @@ If IGNORE-REGION-P is non-nil, only return the entry under point."
 If there is a prefix argument, visit the current entry in the
 browser defined by `browse-url-generic-program'."
   (interactive "P")
-  (let ((entries (elfeed-search-selected)))
+  (let ((buffer (current-buffer))
+        (entries (elfeed-search-selected)))
     (cl-loop for entry in entries
              do (elfeed-untag entry 'unread)
              when (elfeed-entry-link entry)
              do (if use-generic-p
                     (browse-url-generic it)
                   (browse-url it)))
-    (mapc #'elfeed-search-update-entry entries)
-    (unless (or elfeed-search-remain-on-entry (use-region-p))
-      (forward-line))))
+    ;; `browse-url' could have switched to another buffer if eww or another
+    ;; internal browser is used, but the remainder of the functions needs to
+    ;; run in the elfeed buffer.
+    (with-current-buffer buffer
+      (mapc #'elfeed-search-update-entry entries)
+      (unless (or elfeed-search-remain-on-entry (use-region-p))
+        (forward-line)))))
 
 (defun elfeed-search-yank ()
   "Copy the selected feed items to clipboard and kill-ring."
